-- 1.1.4, 8/29/17, toc update for 7.3
-- 1.1.3, 3/28/17, toc update for 7.2
-- 1.1.2, 10/24/16, toc update for 7.1
-- 1.1.1, 9/18/16, fix for potential lua error (line 46: table index is nil)
-- 1.1.0, 7/18/16, complete rewrite for Legion
-- 1.0.7, 6/22/15, toc update for 6.2
-- 1.0.6, 2/24/15, toc update for 6.1
-- 1.0.5, 10/14/14, toc update for 6.0
-- 1.0.4, 9/11/13, toc update for 5.4
-- 1.0.3, 7/11/13, fix for tradeskill addons that change list size (like doublewide ones)
-- 1.0.2, 6/24/13, initial release

local savedvar -- will be TradeSkillBookmarksSaved, the per-character savedvariable

local tsb = CreateFrame("Frame","TradeSkillBookmarks",TradeSkillFrame)
tsb.Buttons = {} -- numerically-indexed table of the buttons (index 0=main button, 1-8=bookmarks)

tsb:SetScript("OnEvent",function(self,event,addon)
	if event=="ADDON_LOADED" and addon=="TradeSkill Bookmarks" then
		tsb:Initialize()
	else
		tsb:Update()
	end
end)
tsb:RegisterEvent("ADDON_LOADED")


function tsb:Initialize()
	tsb:UnregisterEvent("ADDON_LOADED")
	-- setup the per-character savedvar
	if not TradeSkillBookmarksSaved then
		TradeSkillBookmarksSaved = {}
	end
	savedvar = TradeSkillBookmarksSaved
	-- position the parent frame beside the TradeSkillUI's Filter button
	tsb:SetPoint("RIGHT",TradeSkillFrame.FilterButton,"LEFT",-8,1)
	tsb:SetSize(225,24)
	for i=0,8 do
		tsb:CreateButton(i)
	end
	hooksecurefunc(TradeSkillFrame.RecipeList,"RefreshDisplay",tsb.Update)
end

function tsb:Update()
	local tradeSkillID,tradeSkillName = C_TradeSkillUI.GetTradeSkillLine()
	if C_TradeSkillUI.GetOnlyShowUnlearnedRecipes() or not tradeSkillID then
		tsb:Hide()
	else
		if not savedvar[tradeSkillID] then
			savedvar[tradeSkillID] = {}
		end
		local saved = savedvar[tradeSkillID]
		tsb.openSaved = saved
		local selectedRecipe = TradeSkillFrame.RecipeList:GetSelectedRecipeID()
		-- update the bookmark buttons with their recipes
		for i=1,8 do
			local button = tsb.Buttons[i]
			if saved[i] then
				local info = C_TradeSkillUI.GetRecipeInfo(saved[i])
				tsb:SetButtonIcon(button,info and info.icon or "Interface\\Icons\\INV_Misc_QuestionMark")
				button:Show()
			else	
				button:Hide()
			end
		end
		-- update the main button
		local main = tsb.Buttons[0]
		local selectedSaved = tContains(saved,selectedRecipe)
		if not selectedRecipe or (#saved>7 and not selectedSaved) then
			main:Disable()
			main.PlusMinus:Hide()
		else
			main:Enable()
			main.PlusMinus:Show()
			if selectedSaved then
				main.PlusMinus:SetTexture("Interface\\PetBattles\\DeadPetIcon")
			else
				main.PlusMinus:SetTexture("Interface\\PaperDollInfoFrame\\Character-Plus")
			end
		end
		tsb:Show()
	end
end

function tsb:CreateButton(index)
	tsb.Buttons[index] = CreateFrame("Button",nil,tsb)
	local button = tsb.Buttons[index]
	button:SetID(index)
	button:SetSize(24,24)
	button:SetPoint("RIGHT",-(index*25),0)
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
	if index==0 then -- this is the main button with book icon and +/- overlay to add/remove a bookmark
		tsb:SetButtonIcon(button,"Interface\\Icons\\INV_Misc_Book_09")
		button.PlusMinus = button:CreateTexture(nil,"OVERLAY")
		button.PlusMinus:SetSize(12,12)
		button.PlusMinus:SetPoint("BOTTOMRIGHT",-3,3)
	else -- this is one of the eight bookmark buttons
		button:Hide()
	end
	button:SetScript("OnClick",tsb.ButtonOnClick)
	button:SetScript("OnEnter",tsb.ButtonOnEnter)
	button:SetScript("OnLeave",tsb.ButtonOnLeave)
end

function tsb:SetButtonIcon(button,texture)
	button:SetNormalTexture(texture)
	button:SetPushedTexture(texture)
	local pushed = button:GetPushedTexture()
	pushed:ClearAllPoints()
	pushed:SetPoint("TOPLEFT",1,-1)
	pushed:SetPoint("BOTTOMRIGHT",-1,1)
	pushed:SetVertexColor(0.75,0.75,0.75)
end

function tsb:ButtonOnClick()
	local selectedRecipe = TradeSkillFrame.RecipeList:GetSelectedRecipeID()
	local saved = tsb.openSaved
	if self:GetID()==0 then -- main button
		if tContains(saved,selectedRecipe) then -- selected recipe has a bookmark, delete it
			for i=#saved,1,-1 do
				if saved[i]==selectedRecipe then
					tremove(saved,i)
				end
			end
		else -- selected recipe does not have a bookmark, add it
			if #saved<8 then
				tinsert(saved,selectedRecipe)
			end
		end
		tsb:Update()
		GameTooltip:Hide()
	else -- bookmark buttons
		TradeSkillFrame.RecipeList:SelectedAndForceRecipeIDIntoView(saved[self:GetID()])
	end
end

function tsb:ButtonOnEnter()
	GameTooltip:SetOwner(self,"ANCHOR_LEFT")
	if self:GetID()>0 then
		GameTooltip:SetRecipeResultItem(tsb.openSaved[self:GetID()])
	else
		GameTooltip:AddLine(tContains(tsb.openSaved,TradeSkillFrame.RecipeList:GetSelectedRecipeID()) and "Remove Bookmark" or "Add Bookmark",1,1,1)
	end
	GameTooltip:Show()
end

function tsb:ButtonOnLeave()
	GameTooltip:Hide()
end